local metadata = {
"## Interface: FS15 1.4.2.0 1.4.2RC1",
"## Title: LiquideTrigger",
"## Notes: Fluessigkeiten Trigger (in/out) fr FabrikScript",
"## Author: Marhu",
"## Version: 2.0.0-137",
"## Date: 27.03.2016",
"## Web: http://marhu.net"
}

local DebugEbene = 0;
local function getmdata(v) v="## "..v..": "; for i=1,table.getn(metadata) do local _,n=string.find(metadata[i],v);if n then return (string.sub (metadata[i], n+1)); end;end;end;
local function Debug(e,s,...) if e <= DebugEbene then	print((getmdata("Title")).." v"..(getmdata("Version"))..": "..string.format(s,...)); end; end;
local function L(name) local t = getmdata("Title"); return g_i18n:hasText(t.."_"..name) and g_i18n:getText(t.."_"..name) or name; end

local ModDir = g_currentModDirectory;
LiquideTrigger = {}
local LiquideTrigger_mt = Class(LiquideTrigger);
InitObjectClass(LiquideTrigger, "LiquideTrigger");

function LiquideTrigger:new(isServer, isClient)
	local self = {};
	setmetatable(self, LiquideTrigger_mt)
	self.liquideTrailers = {}
	self.isLiquideTankFilling = false;
	self.LiquideTankActivatable = LiquideTankActivatable:new(self)
	self.isClient = isClient
	self.isServer = isServer
	
	return self;
end;

function LiquideTrigger:load(id,tank)
	
	self.nodeId = id;
	self.triggerId = Utils.indexToObject(id, getUserAttribute(id, "triggerIndex"));
    if self.triggerId == nil then -- v2.0.0 UserAttributes muessen bei der Group LiquideTrigger geschrieben werden, nicht mehr beim Trigger selbst.
        self.triggerId = id;
	end
	
	self.Tank = tank;

	addTrigger(self.triggerId, "triggerCallback", self)
		
	local fillTypesStr = Utils.getNoNil(getUserAttribute(self.triggerId, "fillType"),"water")
	fillTypesStr = Utils.getNoNil(getUserAttribute(id, "fillType"),fillTypesStr)
	local fillType = Fillable.fillTypeNameToInt[fillTypesStr]
	if fillType then
		self.fillType = fillType;
	else
		Debug(-1,"ERROR: unknown fillType %s in %s",tostring(fillTypesStr),getName(id));
	end
	self.priceScale = Utils.getNoNil(getUserAttribute(id, "priceScale"), 0)
	self.fillLitersPerSecond = Utils.getNoNil(getUserAttribute(id, "fillLitersPerSecond"), 50);
	
	if self.isClient then
		local SoundFile  = Utils.getFilename("$data/maps/sounds/refuel.wav",  ModDir);	
		self.SourceRefuel = createAudioSource("RefuelSound", SoundFile, 30, 15, 0, 1);
		self.sampleRefuel = getAudioSourceSample(self.SourceRefuel);
		link(id, self.SourceRefuel);
	end
	
	local Name = Utils.getNoNil(getUserAttribute(self.triggerId, "L_Name"), "LiquideTriggerName") -- in der modDesc.xml, text name="TitelDesScriptes_Name"
	Name = Utils.getNoNil(getUserAttribute(id, "L_Name"), Name)
	self.LiquideTankActivatable.liquideTriggerName =  L(Name);
	
	local movingIndex = getUserAttribute(id, "movingIndex")
	if movingIndex then
		local minY, maxY = Utils.getVectorFromString(getUserAttribute(id, "moveMinMaxY"));
		if minY ~= nil and maxY ~= nil then
			local maxAmount = tonumber(getUserAttribute(id, "moveMaxAmount")) or self:liquideTankCapacity();
			Debug(1,"maxAmount %.2f",maxAmount)
			if maxAmount ~= nil then
				self.movingIndex = Utils.indexToObject(id, movingIndex);
				self.moveMinY = minY;
				self.moveMaxY = maxY;
				self.moveMaxAmount = maxAmount;
			end;
		end;
	end;
				
	self.isInputTrigger = Utils.getNoNil(getUserAttribute(id, "isInputTrigger"), true)
	
	self.IsFilling = false
	self.isEnabled = true
	
	g_currentMission:addUpdateable(self);
	return true;
end;

function LiquideTrigger:delete()
	
	removeTrigger(self.triggerId)

end;

function LiquideTrigger:update(dt)
	
	if self.isLiquideTankFilling then
		local disableFilling = true;
		if self.liquideTankFillTrailer then
			if self.isClient and self.liquideTankFillTrailer.setOverloadPipe ~= nil then
				self.liquideTankFillTrailer:setOverloadPipe(self.triggerId);
			end
			if self.isServer then
				local liquideFillLevel = self.liquideTankFillTrailer:getFillLevel(self.fillType)
				local fillLitersPerSecond = self.liquideTankFillTrailer.fillLitersPerSecond or self.liquideTankFillTrailer.fuelFillLitersPerSecond or self.fillLitersPerSecond;
				local delta = fillLitersPerSecond*dt*0.001
				
				if self.isInputTrigger then
					delta = math.min(delta, liquideFillLevel, self:liquideTankCapacity()-self:liquideTankFillLevel())
				else
					delta = math.min(delta, self:liquideTankFillLevel())
				end
				
				if delta > 0 then
					if self.isInputTrigger then
						disableFilling = false
						self.liquideTankFillTrailer:setFillLevel(liquideFillLevel - delta, self.fillType, true);
					else
						self.liquideTankFillTrailer:setFillLevel(liquideFillLevel + delta, self.fillType)
						delta = liquideFillLevel - self.liquideTankFillTrailer:getFillLevel(self.fillType);
						if delta < 0 then disableFilling = false end;
					end;
					self:setLiquideTankFillLevel(self:liquideTankFillLevel() + delta)
				end;
			end;
		end
		if self.isServer and disableFilling then
			self:setIsLiquideTankFilling(false)
		end;
	end
	
	self.elapsed = (self.elapsed or 0) + dt;
	if self.elapsed >= 300 then self:updateTick(self.elapsed); end	
end;

function LiquideTrigger:updateTick(dt)
	self.elapsed = self.elapsed - dt
end;

function LiquideTrigger:addLiquideTrailer(trailer)
	if table.getn(self.liquideTrailers) == 0 then
		g_currentMission:addActivatableObject(self.LiquideTankActivatable)
	end

	table.insert(self.liquideTrailers, trailer)
end

function LiquideTrigger:removeLiquideTrailer(trailer)
	for i = 1, table.getn(self.liquideTrailers), 1 do
		if self.liquideTrailers[i] == trailer then
			table.remove(self.liquideTrailers, i)
			break
		end
	end

	if table.getn(self.liquideTrailers) == 0 then
		g_currentMission:removeActivatableObject(self.LiquideTankActivatable)
	end
	
	if self.isServer and self.liquideTankFillTrailer == trailer then
		self:setIsLiquideTankFilling(false)
	end;
end

function LiquideTrigger:liquideTankCapacity() 
	return self.Tank.getCapacity(self.fillType)
end

function LiquideTrigger:liquideTankFillLevel()
	return self.Tank.getFillLevel(self.fillType)
end

function LiquideTrigger:setLiquideTankFillLevel(lvl)
	self.Tank.setFillLevel(lvl, self.fillType)
end

function LiquideTrigger:updateMoving(amount)
	Debug(5,"%s","updateMoving")
	if self.moveMaxAmount then
		if amount < 0.001 then
			amount = 0;
		end;
		local mover = 0;
		if self.movingIndex then
			mover = self.movingIndex
		elseif self.nodeId then
			mover = self.nodeId
		end
		local x,y,z = getTranslation(mover);
		local y = self.moveMinY + (self.moveMaxY - self.moveMinY)*Utils.clamp(amount, 0, self.moveMaxAmount)/(self.moveMaxAmount);
		Debug(6,"%s","setTranslation(mover)")
		setTranslation(mover, x,y,z);
	end;
end

function LiquideTrigger:setIsLiquideTankFilling(isLiquideTankFilling, trailer, noEventSend)
	LiquideTriggerSetIsFillingEvent.sendEvent(self.Tank.Parent, self.FS_LiquideTriggerId, isLiquideTankFilling, trailer, noEventSend)
	if self.isLiquideTankFilling ~= isLiquideTankFilling then
		self.isLiquideTankFilling = isLiquideTankFilling
		self.liquideTankFillTrailer = trailer
	end

	if self.isClient and self.sampleRefuel ~= nil then
		if isLiquideTankFilling then
			playSample(self.sampleRefuel,0,1,0)
		else
			stopSample(self.sampleRefuel)
		end
	end

	return 
end

function LiquideTrigger:triggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
	local trailer = g_currentMission.objectToTrailer[otherShapeId]

	if trailer ~= nil and trailer.fillTypes ~= nil then
		if onEnter then
			self:addLiquideTrailer(trailer)
		else
			self:removeLiquideTrailer(trailer)
		end
	end
end

LiquideTankActivatable = {}
local LiquideTankActivatable_mt = Class(LiquideTankActivatable)
function LiquideTankActivatable:new(Trigger)
	local self = {}

	setmetatable(self, LiquideTankActivatable_mt)

	self.Trigger = Trigger
	self.activateText = "unknown"
	self.currentTrailer = nil

	return self
end
function LiquideTankActivatable:getIsActivatable()
	self.currentTrailer = nil

	if self.Trigger.isInputTrigger then
		if self.Trigger:liquideTankCapacity() <= self.Trigger:liquideTankFillLevel() then
			Debug(10,"%s","getIsActivatable Capacity < ")
			return false
		end
	elseif self.Trigger:liquideTankFillLevel() <= 0 then
		Debug(10,"%s","getIsActivatable liquideTankFillLevel < ")
		return false
	end

	for _, trailer in pairs(self.Trigger.liquideTrailers) do
		if trailer.getIsActiveForInput(trailer) then
			if self.Trigger.isInputTrigger then
				if 0 < trailer.getFillLevel(trailer, self.Trigger.fillType) then
					self.currentTrailer = trailer
					self.updateActivateText(self)
					Debug(10,"%s","getIsActivatable trailer.getFillLevel ")	
					return true
				end
			elseif trailer:allowFillType(self.Trigger.fillType, false) then
				self.currentTrailer = trailer
				self.updateActivateText(self)
				Debug(10,"%s","getIsActivatable trailer.allowFillType ")	
				return true
			end
		end
	end
	Debug(10,"%s","getIsActivatable false")
	return false
end
function LiquideTankActivatable:onActivateObject()
	self.Trigger:setIsLiquideTankFilling(not self.Trigger.isLiquideTankFilling, self.currentTrailer)
	self.updateActivateText(self)
	g_currentMission:addActivatableObject(self)
end
function LiquideTankActivatable:drawActivate()
end
function LiquideTankActivatable:updateActivateText()
	local typeDesc = self.Trigger.isInputTrigger and self.liquideTriggerName or (self.currentTrailer and self.currentTrailer.typeDesc or "unknown");
	if self.Trigger.isLiquideTankFilling then
		self.activateText = string.format(g_i18n:getText("stop_refill_OBJECT"), (typeDesc or "unknown"))
	else
		self.activateText = string.format(g_i18n:getText("refill_OBJECT"), (typeDesc or "unknown"))
	end
end

LiquideTriggerSetIsFillingEvent = {}
LiquideTriggerSetIsFillingEvent_mt = Class(LiquideTriggerSetIsFillingEvent, Event)
InitEventClass(LiquideTriggerSetIsFillingEvent, "LiquideTriggerSetIsFillingEvent")
function LiquideTriggerSetIsFillingEvent:emptyNew()
	local self = Event:new(LiquideTriggerSetIsFillingEvent_mt)
	return self
end
function LiquideTriggerSetIsFillingEvent:new(object, FS_LiquideTriggerId, isFilling, trailer)
	local self = LiquideTriggerSetIsFillingEvent:emptyNew()
	self.object = object
	self.FS_LiquideTriggerId = FS_LiquideTriggerId;
	self.isFilling = isFilling
	self.trailer = trailer
	return self
end
function LiquideTriggerSetIsFillingEvent:readStream(streamId, connection)
	self.object = networkGetObject(streamReadInt32(streamId))
	self.FS_LiquideTriggerId = streamReadInt32(streamId);
	self.isFilling = streamReadBool(streamId)

	if self.isFilling then
		self.trailer = networkGetObject(streamReadInt32(streamId))
	end

	self:run(connection)
end
function LiquideTriggerSetIsFillingEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.object))
	streamWriteInt32(streamId, self.FS_LiquideTriggerId)
	streamWriteBool(streamId, self.isFilling)

	if self.isFilling then
		streamWriteInt32(streamId, networkGetObjectId(self.trailer))
	end
end
function LiquideTriggerSetIsFillingEvent:run(connection)
	if not connection:getIsServer() then
		g_server:broadcastEvent(self, false, connection, self.object)
	end
	if self.object ~= nil then
		self.object:setIsLiquideTankFilling(self.FS_LiquideTriggerId, self.isFilling, self.trailer, true)
		Debug(5,"%s","self.object ~= nil")
	else
		Debug(5,"%s","self.object == nil")
	end;
end
function LiquideTriggerSetIsFillingEvent.sendEvent(object, FS_LiquideTriggerId, isFilling, trailer, noEventSend)
	if (noEventSend == nil or noEventSend == false) then
		if g_server ~= nil then
			g_server:broadcastEvent(LiquideTriggerSetIsFillingEvent:new(object, FS_LiquideTriggerId, isFilling, trailer), nil, nil, object)
		else
			g_client:getServerConnection():sendEvent(LiquideTriggerSetIsFillingEvent:new(object, FS_LiquideTriggerId, isFilling, trailer))
		end
	end
end